
WEBAPP_ACTIVE_STATE_DISABLED = "disabled"
WEBAPP_ACTIVE_STATE_CURTAINED = "curtained"
WEBAPP_ACTIVE_STATE_ENABLED = "enabled"

WEBAPP_ACTIVE_STATES = (
  (WEBAPP_ACTIVE_STATE_DISABLED, "Disabled"),
  (WEBAPP_ACTIVE_STATE_CURTAINED, "Curtained"),
  (WEBAPP_ACTIVE_STATE_ENABLED, "Enabled"),
)
